@@ -7,8 +7,8 @@ from resotolib.baseresources import *
 from resotolib.graph import Graph
 from resotolib.utils import make_valid_timestamp
 from .utils import aws_client, aws_resource, tags_as_dict
-from typing import ClassVar, Any, Optional, Union
-from dataclasses import dataclass
+from typing import ClassVar, Any, Optional, Union, List, Dict
+from attrs import define
 from resotolib.logger import log
 
 
@@ -16,16 +16,19 @@ default_ctime = make_valid_timestamp(date(2006, 3, 19))  # AWS public launch dat
 
 
 # derived from https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSAccount(BaseAccount):
     kind: ClassVar[str] = "aws_account"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_region"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_region"],
+            "delete": [],
+        }
     }
 
     account_alias: Optional[str] = ""
     role: Optional[str] = None
+    profile: Optional[str] = None
     users: Optional[int] = 0
     groups: Optional[int] = 0
     account_mfa_enabled: Optional[int] = 0
@@ -52,53 +55,56 @@ class AWSAccount(BaseAccount):
         return False
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSRegion(BaseRegion):
     kind: ClassVar[str] = "aws_region"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [
-            "aws_vpc_quota",
-            "aws_vpc_peering_connection",
-            "aws_vpc_endpoint",
-            "aws_vpc",
-            "aws_s3_bucket_quota",
-            "aws_s3_bucket",
-            "aws_rds_instance",
-            "aws_iam_server_certificate_quota",
-            "aws_iam_server_certificate",
-            "aws_iam_role",
-            "aws_iam_policy",
-            "aws_iam_instance_profile",
-            "aws_iam_group",
-            "aws_elb_quota",
-            "aws_elb",
-            "aws_eks_cluster",
-            "aws_ec2_volume_type",
-            "aws_ec2_volume",
-            "aws_iam_user",
-            "aws_ec2_subnet",
-            "aws_ec2_snapshot",
-            "aws_ec2_security_group",
-            "aws_ec2_route_table",
-            "aws_ec2_network_interface",
-            "aws_ec2_network_acl",
-            "aws_ec2_nat_gateway",
-            "aws_ec2_keypair",
-            "aws_ec2_internet_gateway_quota",
-            "aws_ec2_internet_gateway",
-            "aws_ec2_instance_type",
-            "aws_ec2_instance_quota",
-            "aws_ec2_instance",
-            "aws_ec2_elastic_ip",
-            "aws_cloudwatch_alarm",
-            "aws_cloudformation_stack",
-            "aws_cloudformation_stack_set",
-            "aws_autoscaling_group",
-            "aws_alb_target_group",
-            "aws_alb_quota",
-            "aws_alb",
-        ],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [
+                "aws_vpc_quota",
+                "aws_vpc_peering_connection",
+                "aws_vpc_endpoint",
+                "aws_vpc",
+                "aws_s3_bucket_quota",
+                "aws_s3_bucket",
+                "aws_rds_instance",
+                "aws_iam_server_certificate_quota",
+                "aws_iam_server_certificate",
+                "aws_iam_role",
+                "aws_iam_policy",
+                "aws_iam_instance_profile",
+                "aws_iam_group",
+                "aws_elb_quota",
+                "aws_elb",
+                "aws_eks_cluster",
+                "aws_ec2_volume_type",
+                "aws_ec2_volume",
+                "aws_iam_user",
+                "aws_ec2_subnet",
+                "aws_ec2_snapshot",
+                "aws_ec2_security_group",
+                "aws_ec2_route_table",
+                "aws_ec2_network_interface",
+                "aws_ec2_network_acl",
+                "aws_ec2_nat_gateway",
+                "aws_ec2_keypair",
+                "aws_ec2_internet_gateway_quota",
+                "aws_ec2_internet_gateway",
+                "aws_ec2_instance_type",
+                "aws_ec2_instance_quota",
+                "aws_ec2_instance",
+                "aws_ec2_elastic_ip",
+                "aws_cloudwatch_alarm",
+                "aws_cloudformation_stack",
+                "aws_cloudformation_stack_set",
+                "aws_autoscaling_group",
+                "aws_alb_target_group",
+                "aws_alb_quota",
+                "aws_alb",
+                "aws_lambda_function",
+            ],
+            "delete": [],
+        }
     }
     ctime: Optional[datetime] = default_ctime
 
@@ -106,7 +112,7 @@ class AWSRegion(BaseRegion):
         return False
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSResource:
     kind: ClassVar[str] = "aws_resource"
     arn: Optional[str] = None
@@ -115,60 +121,51 @@ class AWSResource:
         return False
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2InstanceType(AWSResource, BaseInstanceType):
     kind: ClassVar[str] = "aws_ec2_instance_type"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_instance"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_instance"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2InstanceQuota(AWSResource, BaseInstanceQuota):
     kind: ClassVar[str] = "aws_ec2_instance_quota"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_instance_type"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_instance_type"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2Instance(AWSResource, BaseInstance):
     kind: ClassVar[str] = "aws_ec2_instance"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [
-            "aws_ec2_volume",
-            "aws_ec2_network_interface",
-            "aws_ec2_keypair",
-            "aws_ec2_elastic_ip",
-            "aws_cloudwatch_alarm",
-        ],
-        "delete": [
-            "aws_elb",
-            "aws_autoscaling_group",
-            "aws_alb_target_group",
-            "aws_cloudwatch_alarm",
-        ],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [
+                "aws_ec2_volume",
+                "aws_ec2_network_interface",
+                "aws_ec2_keypair",
+                "aws_ec2_elastic_ip",
+                "aws_cloudwatch_alarm",
+            ],
+            "delete": [
+                "aws_elb",
+                "aws_autoscaling_group",
+                "aws_alb_target_group",
+                "aws_cloudwatch_alarm",
+            ],
+        }
     }
 
-    instance_status_map: ClassVar[Dict[str, InstanceStatus]] = {
-        "pending": InstanceStatus.BUSY,
-        "running": InstanceStatus.RUNNING,
-        "shutting-down": InstanceStatus.BUSY,
-        "terminated": InstanceStatus.TERMINATED,
-        "stopping": InstanceStatus.BUSY,
-        "stopped": InstanceStatus.STOPPED,
-        "busy": InstanceStatus.BUSY,
-    }
-
-    def _instance_status_setter(self, value: str) -> None:
-        self._instance_status = self.instance_status_map.get(value, InstanceStatus.UNKNOWN)
-        if self._instance_status == InstanceStatus.TERMINATED:
-            self._cleaned = True
-
     def delete(self, graph: Graph) -> bool:
-        if self.instance_status == InstanceStatus.TERMINATED.value:
+        if self.instance_status == InstanceStatus.TERMINATED:
             log.debug(
                 (
                     f"AWS EC2 Instance {self.dname} in"
@@ -197,17 +194,44 @@ class AWSEC2Instance(AWSResource, BaseInstance):
         return True
 
 
-AWSEC2Instance.instance_status = property(
-    AWSEC2Instance._instance_status_getter, AWSEC2Instance._instance_status_setter
-)
-
-
-@dataclass(eq=False)
-class AWSEC2KeyPair(AWSResource, BaseKeyPair):
-    kind: ClassVar[str] = "aws_ec2_keypair"
+@define(eq=False, slots=False)
+class AWSLambdaFunction(AWSResource, BaseServerlessFunction):
+    kind: ClassVar[str] = "aws_lambda_function"
     successor_kinds: ClassVar[Dict[str, List[str]]] = {
         "default": [],
-        "delete": ["aws_ec2_instance"],
+        "delete": [],
+    }
+
+    role: Optional[str] = None
+    runtime: Optional[str] = None
+    code_size: Optional[int] = None
+    memory_size: Optional[int] = None
+    kms_key_arn: Optional[str] = None
+
+    def delete(self, graph: Graph) -> bool:
+        client = aws_client(self, "lambda", graph)
+        client.delete_function(FunctionName=self.id)
+        return True
+
+    def update_tag(self, key, value) -> bool:
+        client = aws_client(self, "lambda")
+        client.tag_resource(Resource=self.id, Tags={key: value})
+        return True
+
+    def delete_tag(self, key) -> bool:
+        client = aws_client(self, "lambda")
+        client.untag_resource(Resource=self.id, TagKeys=[key])
+        return True
+
+
+@define(eq=False, slots=False)
+class AWSEC2KeyPair(AWSResource, BaseKeyPair):
+    kind: ClassVar[str] = "aws_ec2_keypair"
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [],
+            "delete": ["aws_ec2_instance"],
+        }
     }
 
     def delete(self, graph: Graph) -> bool:
@@ -226,21 +250,25 @@ class AWSEC2KeyPair(AWSResource, BaseKeyPair):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2VolumeType(AWSResource, BaseVolumeType):
     kind: ClassVar[str] = "aws_ec2_volume_type"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_volume"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_volume"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2Volume(AWSResource, BaseVolume):
     kind: ClassVar[str] = "aws_ec2_volume"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_snapshot"],
-        "delete": ["aws_ec2_instance"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_snapshot"],
+            "delete": ["aws_ec2_instance"],
+        }
     }
 
     volume_kms_key_id: Optional[str] = None
@@ -248,19 +276,6 @@ class AWSEC2Volume(AWSResource, BaseVolume):
     volume_outpost_arn: Optional[str] = None
     volume_snapshot_id: Optional[str] = None
 
-    volume_status_map: ClassVar[Dict[str, VolumeStatus]] = {
-        "creating": VolumeStatus.BUSY,
-        "available": VolumeStatus.AVAILABLE,
-        "in-use": VolumeStatus.IN_USE,
-        "deleting": VolumeStatus.BUSY,
-        "deleted": VolumeStatus.DELETED,
-        "error": VolumeStatus.ERROR,
-        "busy": VolumeStatus.BUSY,
-    }
-
-    def _volume_status_setter(self, value: str) -> None:
-        self._volume_status = self.volume_status_map.get(value, VolumeStatus.UNKNOWN)
-
     def delete(
         self,
         graph: Graph,
@@ -327,10 +342,7 @@ class AWSEC2Volume(AWSResource, BaseVolume):
         return True
 
 
-AWSEC2Volume.volume_status = property(AWSEC2Volume._volume_status_getter, AWSEC2Volume._volume_status_setter)
-
-
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2Snapshot(AWSResource, BaseSnapshot):
     kind: ClassVar[str] = "aws_ec2_snapshot"
 
@@ -351,26 +363,30 @@ class AWSEC2Snapshot(AWSResource, BaseSnapshot):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2Subnet(AWSResource, BaseSubnet):
     kind: ClassVar[str] = "aws_ec2_subnet"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [
-            "aws_vpc_endpoint",
-            "aws_rds_instance",
-            "aws_elb",
-            "aws_ec2_network_interface",
-            "aws_ec2_network_acl",
-            "aws_ec2_nat_gateway",
-            "aws_alb",
-        ],
-        "delete": [
-            "aws_vpc_endpoint",
-            "aws_rds_instance",
-            "aws_elb",
-            "aws_ec2_network_interface",
-            "aws_alb",
-        ],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [
+                "aws_vpc_endpoint",
+                "aws_rds_instance",
+                "aws_elb",
+                "aws_ec2_network_interface",
+                "aws_ec2_network_acl",
+                "aws_ec2_nat_gateway",
+                "aws_alb",
+                "aws_lambda_function",
+            ],
+            "delete": [
+                "aws_vpc_endpoint",
+                "aws_rds_instance",
+                "aws_elb",
+                "aws_ec2_network_interface",
+                "aws_alb",
+                "aws_lambda_function",
+            ],
+        }
     }
 
     def delete(self, graph: Graph) -> bool:
@@ -390,12 +406,14 @@ class AWSEC2Subnet(AWSResource, BaseSubnet):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2ElasticIP(AWSResource, BaseIPAddress):
     kind: ClassVar[str] = "aws_ec2_elastic_ip"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [],
-        "delete": ["aws_ec2_network_interface", "aws_ec2_instance"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [],
+            "delete": ["aws_ec2_network_interface", "aws_ec2_instance"],
+        }
     }
 
     instance_id: Optional[str] = None
@@ -431,38 +449,42 @@ class AWSEC2ElasticIP(AWSResource, BaseIPAddress):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSVPC(AWSResource, BaseNetwork):
     kind: ClassVar[str] = "aws_vpc"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [
-            "aws_vpc_peering_connection",
-            "aws_vpc_endpoint",
-            "aws_rds_instance",
-            "aws_elb",
-            "aws_ec2_subnet",
-            "aws_ec2_security_group",
-            "aws_ec2_route_table",
-            "aws_ec2_network_interface",
-            "aws_ec2_network_acl",
-            "aws_ec2_nat_gateway",
-            "aws_ec2_internet_gateway",
-            "aws_alb_target_group",
-        ],
-        "delete": [
-            "aws_vpc_peering_connection",
-            "aws_vpc_endpoint",
-            "aws_rds_instance",
-            "aws_elb",
-            "aws_ec2_subnet",
-            "aws_ec2_security_group",
-            "aws_ec2_route_table",
-            "aws_ec2_network_interface",
-            "aws_ec2_network_acl",
-            "aws_ec2_nat_gateway",
-            "aws_ec2_internet_gateway",
-            "aws_alb_target_group",
-        ],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [
+                "aws_vpc_peering_connection",
+                "aws_vpc_endpoint",
+                "aws_rds_instance",
+                "aws_elb",
+                "aws_ec2_subnet",
+                "aws_ec2_security_group",
+                "aws_ec2_route_table",
+                "aws_ec2_network_interface",
+                "aws_ec2_network_acl",
+                "aws_ec2_nat_gateway",
+                "aws_ec2_internet_gateway",
+                "aws_alb_target_group",
+                "aws_lambda_function",
+            ],
+            "delete": [
+                "aws_vpc_peering_connection",
+                "aws_vpc_endpoint",
+                "aws_rds_instance",
+                "aws_elb",
+                "aws_ec2_subnet",
+                "aws_ec2_security_group",
+                "aws_ec2_route_table",
+                "aws_ec2_network_interface",
+                "aws_ec2_network_acl",
+                "aws_ec2_nat_gateway",
+                "aws_ec2_internet_gateway",
+                "aws_alb_target_group",
+                "aws_lambda_function",
+            ],
+        }
     }
     is_default: bool = False
 
@@ -490,16 +512,18 @@ class AWSVPC(AWSResource, BaseNetwork):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSVPCQuota(AWSResource, BaseNetworkQuota):
     kind: ClassVar[str] = "aws_vpc_quota"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_vpc"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_vpc"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSS3Bucket(AWSResource, BaseBucket):
     kind: ClassVar[str] = "aws_s3_bucket"
 
@@ -557,21 +581,25 @@ class AWSS3Bucket(AWSResource, BaseBucket):
         self.tags = self.get_tags()
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSS3BucketQuota(AWSResource, BaseBucketQuota):
     kind: ClassVar[str] = "aws_s3_bucket_quota"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_s3_bucket"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_s3_bucket"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSELB(AWSResource, BaseLoadBalancer):
     kind: ClassVar[str] = "aws_elb"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_instance"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_instance"],
+            "delete": [],
+        }
     }
 
     def delete(self, graph: Graph) -> bool:
@@ -591,12 +619,14 @@ class AWSELB(AWSResource, BaseLoadBalancer):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSALB(AWSResource, BaseLoadBalancer):
     kind: ClassVar[str] = "aws_alb"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_iam_server_certificate", "aws_alb_target_group"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_iam_server_certificate", "aws_alb_target_group"],
+            "delete": [],
+        }
     }
 
     def delete(self, graph: Graph) -> bool:
@@ -616,12 +646,14 @@ class AWSALB(AWSResource, BaseLoadBalancer):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSALBTargetGroup(AWSResource, BaseResource):
     kind: ClassVar[str] = "aws_alb_target_group"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_instance"],
-        "delete": ["aws_alb"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_instance"],
+            "delete": ["aws_alb"],
+        }
     }
 
     target_type: str = ""
@@ -643,25 +675,29 @@ class AWSALBTargetGroup(AWSResource, BaseResource):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSELBQuota(AWSResource, BaseLoadBalancerQuota):
     kind: ClassVar[str] = "aws_elb_quota"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_elb"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_elb"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSALBQuota(AWSResource, BaseLoadBalancerQuota):
     kind: ClassVar[str] = "aws_alb_quota"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_alb"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_alb"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2InternetGateway(AWSResource, BaseGateway):
     kind: ClassVar[str] = "aws_ec2_internet_gateway"
 
@@ -693,12 +729,14 @@ class AWSEC2InternetGateway(AWSResource, BaseGateway):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2NATGateway(AWSResource, BaseGateway):
     kind: ClassVar[str] = "aws_ec2_nat_gateway"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_network_interface"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_network_interface"],
+            "delete": [],
+        }
     }
 
     nat_gateway_status: str = ""
@@ -719,26 +757,35 @@ class AWSEC2NATGateway(AWSResource, BaseGateway):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2InternetGatewayQuota(AWSResource, BaseGatewayQuota):
     kind: ClassVar[str] = "aws_ec2_internet_gateway_quota"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_internet_gateway"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_internet_gateway"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2SecurityGroup(AWSResource, BaseSecurityGroup):
     kind: ClassVar[str] = "aws_ec2_security_group"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [
-            "aws_vpc_endpoint",
-            "aws_rds_instance",
-            "aws_elb",
-            "aws_ec2_network_interface",
-        ],
-        "delete": ["aws_vpc_endpoint", "aws_rds_instance"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [
+                "aws_vpc_endpoint",
+                "aws_rds_instance",
+                "aws_elb",
+                "aws_ec2_network_interface",
+                "aws_lambda_function",
+            ],
+            "delete": [
+                "aws_vpc_endpoint",
+                "aws_rds_instance",
+                "aws_lambda_function",
+            ],
+        }
     }
 
     def pre_delete(self, graph: Graph) -> bool:
@@ -784,12 +831,14 @@ class AWSEC2SecurityGroup(AWSResource, BaseSecurityGroup):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2RouteTable(AWSResource, BaseRoutingTable):
     kind: ClassVar[str] = "aws_ec2_route_table"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_vpc_endpoint"],
-        "delete": ["aws_vpc_endpoint"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_vpc_endpoint"],
+            "delete": ["aws_vpc_endpoint"],
+        }
     }
 
     def pre_delete(self, graph: Graph) -> bool:
@@ -820,7 +869,7 @@ class AWSEC2RouteTable(AWSResource, BaseRoutingTable):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSVPCPeeringConnection(AWSResource, BasePeeringConnection):
     kind: ClassVar[str] = "aws_vpc_peering_connection"
     vpc_peering_connection_status: str = ""
@@ -841,12 +890,14 @@ class AWSVPCPeeringConnection(AWSResource, BasePeeringConnection):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSVPCEndpoint(AWSResource, BaseEndpoint):
     kind: ClassVar[str] = "aws_vpc_endpoint"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_network_interface"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_network_interface"],
+            "delete": [],
+        }
     }
     vpc_endpoint_type: str = ""
     vpc_endpoint_status: str = ""
@@ -867,12 +918,14 @@ class AWSVPCEndpoint(AWSResource, BaseEndpoint):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2NetworkAcl(AWSResource, BaseNetworkAcl):
     kind: ClassVar[str] = "aws_ec2_network_acl"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [],
-        "delete": ["aws_ec2_subnet"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [],
+            "delete": ["aws_ec2_subnet"],
+        }
     }
     is_default: bool = False
 
@@ -892,12 +945,14 @@ class AWSEC2NetworkAcl(AWSResource, BaseNetworkAcl):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEC2NetworkInterface(AWSResource, BaseNetworkInterface):
     kind: ClassVar[str] = "aws_ec2_network_interface"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_elastic_ip"],
-        "delete": ["aws_vpc_endpoint", "aws_ec2_nat_gateway", "aws_ec2_instance"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_elastic_ip"],
+            "delete": ["aws_vpc_endpoint", "aws_ec2_nat_gateway", "aws_ec2_instance"],
+        }
     }
 
     def delete(self, graph: Graph) -> bool:
@@ -917,20 +972,22 @@ class AWSEC2NetworkInterface(AWSResource, BaseNetworkInterface):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSRDSInstance(AWSResource, BaseDatabase):
     kind: ClassVar[str] = "aws_rds_instance"
     volume_kms_key_id: Optional[str] = None
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMUser(AWSResource, BaseUser):
     kind: ClassVar[str] = "aws_iam_user"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_iam_access_key"],
-        "delete": ["aws_iam_policy"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_iam_access_key"],
+            "delete": ["aws_iam_policy"],
+        }
     }
-    user_policies: List = field(default_factory=list)
+    user_policies: List = field(factory=list)
 
     def pre_delete(self, graph: Graph) -> bool:
         iam = aws_resource(self, "iam", graph)
@@ -960,14 +1017,16 @@ class AWSIAMUser(AWSResource, BaseUser):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMGroup(AWSResource, BaseGroup):
     kind: ClassVar[str] = "aws_iam_group"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_iam_user"],
-        "delete": ["aws_iam_policy"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_iam_user"],
+            "delete": ["aws_iam_policy"],
+        }
     }
-    group_policies: List = field(default_factory=list)
+    group_policies: List = field(factory=list)
 
     def pre_delete(self, graph: Graph) -> bool:
         iam = aws_resource(self, "iam", graph)
@@ -997,15 +1056,17 @@ class AWSIAMGroup(AWSResource, BaseGroup):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMRole(AWSResource, BaseRole):
     kind: ClassVar[str] = "aws_iam_role"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_iam_policy", "aws_iam_instance_profile"],
-        "delete": ["aws_iam_policy", "aws_iam_instance_profile", "aws_eks_cluster"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_iam_policy", "aws_iam_instance_profile", "aws_eks_cluster", "aws_lambda_function"],
+            "delete": ["aws_iam_policy", "aws_iam_instance_profile", "aws_eks_cluster", "aws_lambda_function"],
+        }
     }
 
-    role_policies: List = field(default_factory=list)
+    role_policies: List = field(factory=list)
 
     def pre_delete(self, graph: Graph) -> bool:
         iam = aws_resource(self, "iam", graph)
@@ -1035,12 +1096,14 @@ class AWSIAMRole(AWSResource, BaseRole):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMPolicy(AWSResource, BasePolicy):
     kind: ClassVar[str] = "aws_iam_policy"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_iam_user", "aws_iam_group"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_iam_user", "aws_iam_group"],
+            "delete": [],
+        }
     }
 
     def delete(self, graph: Graph) -> bool:
@@ -1050,12 +1113,14 @@ class AWSIAMPolicy(AWSResource, BasePolicy):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMInstanceProfile(AWSResource, BaseInstanceProfile):
     kind: ClassVar[str] = "aws_iam_instance_profile"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_instance"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_instance"],
+            "delete": [],
+        }
     }
 
     def pre_delete(self, graph: Graph) -> bool:
@@ -1076,7 +1141,7 @@ class AWSIAMInstanceProfile(AWSResource, BaseInstanceProfile):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMAccessKey(AWSResource, BaseAccessKey):
     kind: ClassVar[str] = "aws_iam_access_key"
 
@@ -1091,12 +1156,14 @@ class AWSIAMAccessKey(AWSResource, BaseAccessKey):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMServerCertificate(AWSResource, BaseCertificate):
     kind: ClassVar[str] = "aws_iam_server_certificate"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [],
-        "delete": ["aws_alb"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [],
+            "delete": ["aws_alb"],
+        }
     }
     path: str = None
 
@@ -1107,16 +1174,18 @@ class AWSIAMServerCertificate(AWSResource, BaseCertificate):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSIAMServerCertificateQuota(AWSResource, BaseCertificateQuota):
     kind: ClassVar[str] = "aws_iam_server_certificate_quota"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_iam_server_certificate"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_iam_server_certificate"],
+            "delete": [],
+        }
     }
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSCloudFormationStack(AWSResource, BaseStack):
     kind: ClassVar[str] = "aws_cloudformation_stack"
 
@@ -1184,12 +1253,14 @@ class AWSCloudFormationStack(AWSResource, BaseStack):
         return stack
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEKSCluster(AWSResource, BaseResource):
     kind: ClassVar[str] = "aws_eks_cluster"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_iam_role", "aws_eks_nodegroup"],
-        "delete": ["aws_eks_nodegroup"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_eks_nodegroup"],
+            "delete": ["aws_eks_nodegroup"],
+        }
     }
     cluster_status: str = ""
     cluster_endpoint: str = ""
@@ -1210,12 +1281,14 @@ class AWSEKSCluster(AWSResource, BaseResource):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSEKSNodegroup(AWSResource, BaseResource):
     kind: ClassVar[str] = "aws_eks_nodegroup"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_autoscaling_group"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_autoscaling_group"],
+            "delete": [],
+        }
     }
 
     cluster_name: str = ""
@@ -1237,12 +1310,14 @@ class AWSEKSNodegroup(AWSResource, BaseResource):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSAutoScalingGroup(AWSResource, BaseAutoScalingGroup):
     kind: ClassVar[str] = "aws_autoscaling_group"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_ec2_instance"],
-        "delete": ["aws_eks_nodegroup"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_ec2_instance"],
+            "delete": ["aws_eks_nodegroup"],
+        }
     }
 
     def delete(self, graph: Graph, force_delete: bool = True) -> bool:
@@ -1281,24 +1356,26 @@ class AWSAutoScalingGroup(AWSResource, BaseAutoScalingGroup):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSCloudwatchAlarm(AWSResource, BaseResource):
     kind: ClassVar[str] = "aws_cloudwatch_alarm"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [],
+            "delete": [],
+        }
     }
 
     actions_enabled: bool = False
     alarm_description: Optional[str] = None
-    alarm_actions: Optional[List] = field(default_factory=list)
+    alarm_actions: Optional[List] = field(factory=list)
     comparison_operator: Optional[str] = None
-    dimensions: Optional[List] = field(default_factory=list)
+    dimensions: Optional[List] = field(factory=list)
     evaluation_periods: Optional[int] = 0
-    insufficient_data_actions: Optional[List] = field(default_factory=list)
+    insufficient_data_actions: Optional[List] = field(factory=list)
     metric_name: Optional[str] = None
     namespace: Optional[str] = None
-    ok_actions: Optional[List] = field(default_factory=list)
+    ok_actions: Optional[List] = field(factory=list)
     period: Optional[int] = 0
     state_value: Optional[str] = None
     statistic: Optional[str] = None
@@ -1321,20 +1398,20 @@ class AWSCloudwatchAlarm(AWSResource, BaseResource):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSCloudFormationStackSet(AWSResource, BaseResource):
     kind: ClassVar[str] = "aws_cloudformation_stack_set"
     description: Optional[str] = None
     stack_set_status: Optional[str] = None
-    stack_set_parameters: Dict = field(default_factory=dict)
-    stack_set_capabilities: Optional[List[str]] = field(default_factory=list)
+    stack_set_parameters: Dict = field(factory=dict)
+    stack_set_capabilities: Optional[List[str]] = field(factory=list)
     stack_set_administration_role_arn: Optional[str] = None
     stack_set_execution_role_name: Optional[str] = None
-    stack_set_drift_detection_details: Optional[Dict[str, Any]] = field(default_factory=dict)
+    stack_set_drift_detection_details: Optional[Dict[str, Any]] = field(factory=dict)
     stack_set_last_drift_check_timestamp: Optional[datetime] = None
-    stack_set_auto_deployment: Optional[Dict[str, bool]] = field(default_factory=dict)
+    stack_set_auto_deployment: Optional[Dict[str, bool]] = field(factory=dict)
     stack_set_permission_model: Optional[str] = None
-    stack_set_organizational_unit_ids: Optional[List[str]] = field(default_factory=list)
+    stack_set_organizational_unit_ids: Optional[List[str]] = field(factory=list)
     stack_set_managed_execution_active: Optional[bool] = None
 
     def delete(self, graph: Graph) -> bool:
@@ -1385,12 +1462,14 @@ class AWSCloudFormationStackSet(AWSResource, BaseResource):
         return True
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSRoute53Zone(AWSResource, BaseDNSZone):
     kind: ClassVar[str] = "aws_route53_zone"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_route53_resource_record_set"],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_route53_resource_record_set"],
+            "delete": [],
+        }
     }
 
     zone_caller_reference: Optional[str] = None
@@ -1399,12 +1478,14 @@ class AWSRoute53Zone(AWSResource, BaseDNSZone):
     zone_linked_service: Optional[Dict[str, str]] = None
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSRoute53ResourceRecordSet(AWSResource, BaseDNSRecordSet):
     kind: ClassVar[str] = "aws_route53_resource_record_set"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": ["aws_route53_resource_record"],
-        "delete": ["aws_route53_resource_record"],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": ["aws_route53_resource_record"],
+            "delete": ["aws_route53_resource_record"],
+        }
     }
     record_set_identifier: Optional[str] = None
     record_region: Optional[str] = None
@@ -1416,10 +1497,12 @@ class AWSRoute53ResourceRecordSet(AWSResource, BaseDNSRecordSet):
     record_traffic_policy_instance_id: Optional[str] = None
 
 
-@dataclass(eq=False)
+@define(eq=False, slots=False)
 class AWSRoute53ResourceRecord(AWSResource, BaseDNSRecord):
     kind: ClassVar[str] = "aws_route53_resource_record"
-    successor_kinds: ClassVar[Dict[str, List[str]]] = {
-        "default": [],
-        "delete": [],
+    reference_kinds: ClassVar[ModelReference] = {
+        "successors": {
+            "default": [],
+            "delete": [],
+        }
     }
